/*
 *	PM - PlantCaretaker
 *	Ialama Nicoleta 336CB
 */
#include <util/delay.h>
#include "lcd.h"
#include <string.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include "ADC_sensors.h"
#include <math.h>

#define LUMINA_SLABA 200
#define NIVEL_APA_SCAZUT 300
#define UMIDITATE_SCAZUTA 8
#define TEMPERATURA_SCAZUTA 10

#define SERIESRESISTOR 10000
#define NOMINAL_RESISTANCE 10000
#define NOMINAL_TEMPERATURE 25
#define BCOEFFICIENT 3950
#define ACOEFICIENT 90
#define CELSIUS 273.15

void ADC_Init()
{
	DDRA = 0x0;		//  ADC port input   
	ADMUX = (1 << REFS0) | (1 << ADC_INIT_CHANNEL);
    ADCSRA = (1 << ADEN) | (1 << ADIE) | (1 << ADSC) | (1 << ADPS0) | (1 << ADPS2);
}

int convert_temperature(int adc_value_temperature) {
	float Resistance;
	Resistance = (1023 / adc_value_temperature) - 1;
	Resistance = SERIESRESISTOR / Resistance;
	float steinhart;
	steinhart = Resistance / NOMINAL_RESISTANCE; // (R/Ro)
	steinhart = log(steinhart); // ln(R/Ro)
	steinhart /= BCOEFFICIENT; // 1/B * ln(R/Ro)
	steinhart += 1.0 / (NOMINAL_TEMPERATURE + 273.15); // + (1/To)
	steinhart = 1.0 / steinhart; // Invert
	steinhart -= CELSIUS + ACOEFICIENT; // conversie la grade celsius
	return steinhart;
}


int main() {
	ADC_Init();		/* initializare ADC */
	TIMER1_init();
	sei(); // Pornesc intreruperile
	LCD_init();
	int adc_value_soil = 0, adc_value_light = 0, adc_value_temperature = 0,
 adc_value_level = 0;
	int temperatura = 0;
	char string[100]; 		
	//LED-uri
	DDRB |= (1 << PB4); // LED albastru - Nivel apa
	PORTB &= ~(1 << PB4);
	DDRB |= (1 << PB5); // LED galben - Lumina
	PORTB &= ~(1 << PB5);
	DDRB |= (1 << PB6); // LED verde - Umiditate sol
	PORTB &= ~(1 << PB6);
	DDRB |= (1 << PB7); // LED rosu - Temperatura
	PORTB &= ~(1 << PB7);
	//Pompa + LED placa
	DDRD |= (1 << PD7); 
	PORTD |= (1 << PD7);

	while(1) {
		const adc_channel_t *channelU = &ADC_channels[0]; // umiditate
		const adc_channel_t *channelL = &ADC_channels[1]; // lumina
		const adc_channel_t *channelN = &ADC_channels[2]; // nivel apa
		const adc_channel_t *channelT = &ADC_channels[3]; // temperatura
		adc_value_soil = ADC_get(channelU->channel);
		adc_value_light = ADC_get(channelL->channel);
		adc_value_level = ADC_get(channelN->channel);
		adc_value_temperature = ADC_get(channelT->channel);
		
		sprintf(string, "Lumina: %d", adc_value_light);
		LCD_writeInstr(LCD_INSTR_clearDisplay);	
		LCD_print(string);
		_delay_ms(700);

		sprintf(string, "Nivel: %d", adc_value_level);
		LCD_writeInstr(LCD_INSTR_clearDisplay);	
		LCD_print(string);
		_delay_ms(700);

		sprintf(string, "Sol: %d", adc_value_soil);
		LCD_writeInstr(LCD_INSTR_clearDisplay);	
		LCD_print(string);
		_delay_ms(700);

		temperatura = (int)convert_temperature(adc_value_temperature);
		sprintf(string, "Temp: %d", temperatura);
		LCD_writeInstr(LCD_INSTR_clearDisplay);	
		LCD_print(string);
		_delay_ms(700);

		if(adc_value_soil < UMIDITATE_SCAZUTA) {
			PORTB |= (1 << PB6); //aprind led
			PORTD &= ~(1 << PD7); //pornesc pompa
		} else {
			PORTB &= ~(1 << PB6); //sting led
			PORTD |= (1 << PD7); //opresc pompa
		}

		if(adc_value_light < LUMINA_SLABA) 
			PORTB |= (1 << PB5);
		else 
			PORTB &= ~(1 << PB5);

		if(adc_value_level < NIVEL_APA_SCAZUT) 
			PORTB |= (1 << PB4);
		else 
			PORTB &= ~(1 << PB4);

		if(temperatura < TEMPERATURA_SCAZUTA)
			PORTB |= (1 << PB7);
		else 
			PORTB &= ~(1 << PB7);	

	}

	return 0;
}

